/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     google/protobuf/struct.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "google/protobuf/struct.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout *const google_protobuf_Struct_submsgs[1] = {
  &google_protobuf_Struct_FieldsEntry_msginit,
};

static const upb_msglayout_field google_protobuf_Struct__fields[1] = {
  {1, UPB_SIZE(0, 0), 0, 0, 11, _UPB_MODE_MAP},
};

const upb_msglayout google_protobuf_Struct_msginit = {
  &google_protobuf_Struct_submsgs[0],
  &google_protobuf_Struct__fields[0],
  UPB_SIZE(8, 8), 1, false, 1, 255,
};

static const upb_msglayout *const google_protobuf_Struct_FieldsEntry_submsgs[1] = {
  &google_protobuf_Value_msginit,
};

static const upb_msglayout_field google_protobuf_Struct_FieldsEntry__fields[2] = {
  {1, UPB_SIZE(0, 0), 0, 0, 9, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(8, 16), 0, 0, 11, _UPB_MODE_SCALAR},
};

const upb_msglayout google_protobuf_Struct_FieldsEntry_msginit = {
  &google_protobuf_Struct_FieldsEntry_submsgs[0],
  &google_protobuf_Struct_FieldsEntry__fields[0],
  UPB_SIZE(16, 32), 2, false, 2, 255,
};

static const upb_msglayout *const google_protobuf_Value_submsgs[2] = {
  &google_protobuf_ListValue_msginit,
  &google_protobuf_Struct_msginit,
};

static const upb_msglayout_field google_protobuf_Value__fields[6] = {
  {1, UPB_SIZE(0, 0), UPB_SIZE(-9, -17), 0, 14, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(0, 0), UPB_SIZE(-9, -17), 0, 1, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(0, 0), UPB_SIZE(-9, -17), 0, 9, _UPB_MODE_SCALAR},
  {4, UPB_SIZE(0, 0), UPB_SIZE(-9, -17), 0, 8, _UPB_MODE_SCALAR},
  {5, UPB_SIZE(0, 0), UPB_SIZE(-9, -17), 1, 11, _UPB_MODE_SCALAR},
  {6, UPB_SIZE(0, 0), UPB_SIZE(-9, -17), 0, 11, _UPB_MODE_SCALAR},
};

const upb_msglayout google_protobuf_Value_msginit = {
  &google_protobuf_Value_submsgs[0],
  &google_protobuf_Value__fields[0],
  UPB_SIZE(16, 32), 6, false, 6, 255,
};

static const upb_msglayout *const google_protobuf_ListValue_submsgs[1] = {
  &google_protobuf_Value_msginit,
};

static const upb_msglayout_field google_protobuf_ListValue__fields[1] = {
  {1, UPB_SIZE(0, 0), 0, 0, 11, _UPB_MODE_ARRAY},
};

const upb_msglayout google_protobuf_ListValue_msginit = {
  &google_protobuf_ListValue_submsgs[0],
  &google_protobuf_ListValue__fields[0],
  UPB_SIZE(8, 8), 1, false, 1, 255,
};

#include "upb/port_undef.inc"

