/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     xds/core/v3/resource.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "xds/core/v3/resource.upb.h"
#include "google/protobuf/any.upb.h"
#include "udpa/annotations/status.upb.h"
#include "xds/core/v3/resource_name.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout *const xds_core_v3_Resource_submsgs[2] = {
  &google_protobuf_Any_msginit,
  &xds_core_v3_ResourceName_msginit,
};

static const upb_msglayout_field xds_core_v3_Resource__fields[3] = {
  {1, UPB_SIZE(12, 24), 1, 1, 11, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(4, 8), 0, 0, 9, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(16, 32), 2, 0, 11, _UPB_MODE_SCALAR},
};

const upb_msglayout xds_core_v3_Resource_msginit = {
  &xds_core_v3_Resource_submsgs[0],
  &xds_core_v3_Resource__fields[0],
  UPB_SIZE(24, 48), 3, false, 3, 255,
};

#include "upb/port_undef.inc"

