/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/service/discovery/v3/discovery.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "envoy/service/discovery/v3/discovery.upb.h"
#include "envoy/config/core/v3/base.upb.h"
#include "google/protobuf/any.upb.h"
#include "google/protobuf/duration.upb.h"
#include "google/rpc/status.upb.h"
#include "udpa/annotations/status.upb.h"
#include "udpa/annotations/versioning.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout *const envoy_service_discovery_v3_DiscoveryRequest_submsgs[2] = {
  &envoy_config_core_v3_Node_msginit,
  &google_rpc_Status_msginit,
};

static const upb_msglayout_field envoy_service_discovery_v3_DiscoveryRequest__fields[6] = {
  {1, UPB_SIZE(4, 8), 0, 0, 9, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(28, 56), 1, 0, 11, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(36, 72), 0, 0, 9, _UPB_MODE_ARRAY},
  {4, UPB_SIZE(12, 24), 0, 0, 9, _UPB_MODE_SCALAR},
  {5, UPB_SIZE(20, 40), 0, 0, 9, _UPB_MODE_SCALAR},
  {6, UPB_SIZE(32, 64), 2, 1, 11, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_service_discovery_v3_DiscoveryRequest_msginit = {
  &envoy_service_discovery_v3_DiscoveryRequest_submsgs[0],
  &envoy_service_discovery_v3_DiscoveryRequest__fields[0],
  UPB_SIZE(40, 80), 6, false, 6, 255,
};

static const upb_msglayout *const envoy_service_discovery_v3_DiscoveryResponse_submsgs[2] = {
  &envoy_config_core_v3_ControlPlane_msginit,
  &google_protobuf_Any_msginit,
};

static const upb_msglayout_field envoy_service_discovery_v3_DiscoveryResponse__fields[6] = {
  {1, UPB_SIZE(4, 8), 0, 0, 9, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(32, 64), 0, 1, 11, _UPB_MODE_ARRAY},
  {3, UPB_SIZE(1, 1), 0, 0, 8, _UPB_MODE_SCALAR},
  {4, UPB_SIZE(12, 24), 0, 0, 9, _UPB_MODE_SCALAR},
  {5, UPB_SIZE(20, 40), 0, 0, 9, _UPB_MODE_SCALAR},
  {6, UPB_SIZE(28, 56), 1, 0, 11, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_service_discovery_v3_DiscoveryResponse_msginit = {
  &envoy_service_discovery_v3_DiscoveryResponse_submsgs[0],
  &envoy_service_discovery_v3_DiscoveryResponse__fields[0],
  UPB_SIZE(40, 80), 6, false, 6, 255,
};

static const upb_msglayout *const envoy_service_discovery_v3_DeltaDiscoveryRequest_submsgs[3] = {
  &envoy_config_core_v3_Node_msginit,
  &envoy_service_discovery_v3_DeltaDiscoveryRequest_InitialResourceVersionsEntry_msginit,
  &google_rpc_Status_msginit,
};

static const upb_msglayout_field envoy_service_discovery_v3_DeltaDiscoveryRequest__fields[7] = {
  {1, UPB_SIZE(20, 40), 1, 0, 11, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(4, 8), 0, 0, 9, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(28, 56), 0, 0, 9, _UPB_MODE_ARRAY},
  {4, UPB_SIZE(32, 64), 0, 0, 9, _UPB_MODE_ARRAY},
  {5, UPB_SIZE(36, 72), 0, 1, 11, _UPB_MODE_MAP},
  {6, UPB_SIZE(12, 24), 0, 0, 9, _UPB_MODE_SCALAR},
  {7, UPB_SIZE(24, 48), 2, 2, 11, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_service_discovery_v3_DeltaDiscoveryRequest_msginit = {
  &envoy_service_discovery_v3_DeltaDiscoveryRequest_submsgs[0],
  &envoy_service_discovery_v3_DeltaDiscoveryRequest__fields[0],
  UPB_SIZE(40, 80), 7, false, 7, 255,
};

static const upb_msglayout_field envoy_service_discovery_v3_DeltaDiscoveryRequest_InitialResourceVersionsEntry__fields[2] = {
  {1, UPB_SIZE(0, 0), 0, 0, 9, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(8, 16), 0, 0, 9, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_service_discovery_v3_DeltaDiscoveryRequest_InitialResourceVersionsEntry_msginit = {
  NULL,
  &envoy_service_discovery_v3_DeltaDiscoveryRequest_InitialResourceVersionsEntry__fields[0],
  UPB_SIZE(16, 32), 2, false, 2, 255,
};

static const upb_msglayout *const envoy_service_discovery_v3_DeltaDiscoveryResponse_submsgs[2] = {
  &envoy_config_core_v3_ControlPlane_msginit,
  &envoy_service_discovery_v3_Resource_msginit,
};

static const upb_msglayout_field envoy_service_discovery_v3_DeltaDiscoveryResponse__fields[6] = {
  {1, UPB_SIZE(4, 8), 0, 0, 9, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(32, 64), 0, 1, 11, _UPB_MODE_ARRAY},
  {4, UPB_SIZE(12, 24), 0, 0, 9, _UPB_MODE_SCALAR},
  {5, UPB_SIZE(20, 40), 0, 0, 9, _UPB_MODE_SCALAR},
  {6, UPB_SIZE(36, 72), 0, 0, 9, _UPB_MODE_ARRAY},
  {7, UPB_SIZE(28, 56), 1, 0, 11, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_service_discovery_v3_DeltaDiscoveryResponse_msginit = {
  &envoy_service_discovery_v3_DeltaDiscoveryResponse_submsgs[0],
  &envoy_service_discovery_v3_DeltaDiscoveryResponse__fields[0],
  UPB_SIZE(40, 80), 6, false, 2, 255,
};

static const upb_msglayout *const envoy_service_discovery_v3_Resource_submsgs[3] = {
  &envoy_service_discovery_v3_Resource_CacheControl_msginit,
  &google_protobuf_Any_msginit,
  &google_protobuf_Duration_msginit,
};

static const upb_msglayout_field envoy_service_discovery_v3_Resource__fields[6] = {
  {1, UPB_SIZE(4, 8), 0, 0, 9, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(20, 40), 1, 1, 11, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(12, 24), 0, 0, 9, _UPB_MODE_SCALAR},
  {4, UPB_SIZE(32, 64), 0, 0, 9, _UPB_MODE_ARRAY},
  {6, UPB_SIZE(24, 48), 2, 2, 11, _UPB_MODE_SCALAR},
  {7, UPB_SIZE(28, 56), 3, 0, 11, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_service_discovery_v3_Resource_msginit = {
  &envoy_service_discovery_v3_Resource_submsgs[0],
  &envoy_service_discovery_v3_Resource__fields[0],
  UPB_SIZE(40, 80), 6, false, 4, 255,
};

static const upb_msglayout_field envoy_service_discovery_v3_Resource_CacheControl__fields[1] = {
  {1, UPB_SIZE(0, 0), 0, 0, 8, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_service_discovery_v3_Resource_CacheControl_msginit = {
  NULL,
  &envoy_service_discovery_v3_Resource_CacheControl__fields[0],
  UPB_SIZE(8, 8), 1, false, 1, 255,
};

#include "upb/port_undef.inc"

