/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/type/v3/http.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef ENVOY_TYPE_V3_HTTP_PROTO_UPB_H_
#define ENVOY_TYPE_V3_HTTP_PROTO_UPB_H_

#include "upb/msg_internal.h"
#include "upb/decode.h"
#include "upb/decode_fast.h"
#include "upb/encode.h"

#include "upb/port_def.inc"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
  envoy_type_v3_HTTP1 = 0,
  envoy_type_v3_HTTP2 = 1,
  envoy_type_v3_HTTP3 = 2
} envoy_type_v3_CodecClientType;


#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port_undef.inc"

#endif  /* ENVOY_TYPE_V3_HTTP_PROTO_UPB_H_ */
