/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/type/v3/http.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "envoy/type/v3/http.upb.h"
#include "udpa/annotations/status.upb.h"

#include "upb/port_def.inc"

#include "upb/port_undef.inc"

