/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/service/status/v3/csds.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "envoy/service/status/v3/csds.upb.h"
#include "envoy/admin/v3/config_dump.upb.h"
#include "envoy/config/core/v3/base.upb.h"
#include "envoy/type/matcher/v3/node.upb.h"
#include "google/api/annotations.upb.h"
#include "google/protobuf/any.upb.h"
#include "google/protobuf/timestamp.upb.h"
#include "envoy/annotations/deprecation.upb.h"
#include "udpa/annotations/status.upb.h"
#include "udpa/annotations/versioning.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout *const envoy_service_status_v3_ClientStatusRequest_submsgs[2] = {
  &envoy_config_core_v3_Node_msginit,
  &envoy_type_matcher_v3_NodeMatcher_msginit,
};

static const upb_msglayout_field envoy_service_status_v3_ClientStatusRequest__fields[2] = {
  {1, UPB_SIZE(8, 16), 0, 1, 11, _UPB_MODE_ARRAY},
  {2, UPB_SIZE(4, 8), 1, 0, 11, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_service_status_v3_ClientStatusRequest_msginit = {
  &envoy_service_status_v3_ClientStatusRequest_submsgs[0],
  &envoy_service_status_v3_ClientStatusRequest__fields[0],
  UPB_SIZE(16, 24), 2, false, 2, 255,
};

static const upb_msglayout *const envoy_service_status_v3_PerXdsConfig_submsgs[5] = {
  &envoy_admin_v3_ClustersConfigDump_msginit,
  &envoy_admin_v3_EndpointsConfigDump_msginit,
  &envoy_admin_v3_ListenersConfigDump_msginit,
  &envoy_admin_v3_RoutesConfigDump_msginit,
  &envoy_admin_v3_ScopedRoutesConfigDump_msginit,
};

static const upb_msglayout_field envoy_service_status_v3_PerXdsConfig__fields[7] = {
  {1, UPB_SIZE(0, 0), 0, 0, 14, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(8, 8), UPB_SIZE(-13, -17), 2, 11, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(8, 8), UPB_SIZE(-13, -17), 0, 11, _UPB_MODE_SCALAR},
  {4, UPB_SIZE(8, 8), UPB_SIZE(-13, -17), 3, 11, _UPB_MODE_SCALAR},
  {5, UPB_SIZE(8, 8), UPB_SIZE(-13, -17), 4, 11, _UPB_MODE_SCALAR},
  {6, UPB_SIZE(8, 8), UPB_SIZE(-13, -17), 1, 11, _UPB_MODE_SCALAR},
  {7, UPB_SIZE(4, 4), 0, 0, 14, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_service_status_v3_PerXdsConfig_msginit = {
  &envoy_service_status_v3_PerXdsConfig_submsgs[0],
  &envoy_service_status_v3_PerXdsConfig__fields[0],
  UPB_SIZE(16, 24), 7, false, 7, 255,
};

static const upb_msglayout *const envoy_service_status_v3_ClientConfig_submsgs[3] = {
  &envoy_config_core_v3_Node_msginit,
  &envoy_service_status_v3_ClientConfig_GenericXdsConfig_msginit,
  &envoy_service_status_v3_PerXdsConfig_msginit,
};

static const upb_msglayout_field envoy_service_status_v3_ClientConfig__fields[3] = {
  {1, UPB_SIZE(4, 8), 1, 0, 11, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(8, 16), 0, 2, 11, _UPB_MODE_ARRAY},
  {3, UPB_SIZE(12, 24), 0, 1, 11, _UPB_MODE_ARRAY},
};

const upb_msglayout envoy_service_status_v3_ClientConfig_msginit = {
  &envoy_service_status_v3_ClientConfig_submsgs[0],
  &envoy_service_status_v3_ClientConfig__fields[0],
  UPB_SIZE(16, 32), 3, false, 3, 255,
};

static const upb_msglayout *const envoy_service_status_v3_ClientConfig_GenericXdsConfig_submsgs[3] = {
  &envoy_admin_v3_UpdateFailureState_msginit,
  &google_protobuf_Any_msginit,
  &google_protobuf_Timestamp_msginit,
};

static const upb_msglayout_field envoy_service_status_v3_ClientConfig_GenericXdsConfig__fields[9] = {
  {1, UPB_SIZE(16, 16), 0, 0, 9, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(24, 32), 0, 0, 9, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(32, 48), 0, 0, 9, _UPB_MODE_SCALAR},
  {4, UPB_SIZE(40, 64), 1, 1, 11, _UPB_MODE_SCALAR},
  {5, UPB_SIZE(44, 72), 2, 2, 11, _UPB_MODE_SCALAR},
  {6, UPB_SIZE(4, 4), 0, 0, 14, _UPB_MODE_SCALAR},
  {7, UPB_SIZE(8, 8), 0, 0, 14, _UPB_MODE_SCALAR},
  {8, UPB_SIZE(48, 80), 3, 0, 11, _UPB_MODE_SCALAR},
  {9, UPB_SIZE(12, 12), 0, 0, 8, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_service_status_v3_ClientConfig_GenericXdsConfig_msginit = {
  &envoy_service_status_v3_ClientConfig_GenericXdsConfig_submsgs[0],
  &envoy_service_status_v3_ClientConfig_GenericXdsConfig__fields[0],
  UPB_SIZE(56, 96), 9, false, 9, 255,
};

static const upb_msglayout *const envoy_service_status_v3_ClientStatusResponse_submsgs[1] = {
  &envoy_service_status_v3_ClientConfig_msginit,
};

static const upb_msglayout_field envoy_service_status_v3_ClientStatusResponse__fields[1] = {
  {1, UPB_SIZE(0, 0), 0, 0, 11, _UPB_MODE_ARRAY},
};

const upb_msglayout envoy_service_status_v3_ClientStatusResponse_msginit = {
  &envoy_service_status_v3_ClientStatusResponse_submsgs[0],
  &envoy_service_status_v3_ClientStatusResponse__fields[0],
  UPB_SIZE(8, 8), 1, false, 1, 255,
};

#include "upb/port_undef.inc"

