/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/config/cluster/v3/outlier_detection.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "envoy/config/cluster/v3/outlier_detection.upb.h"
#include "google/protobuf/duration.upb.h"
#include "google/protobuf/wrappers.upb.h"
#include "udpa/annotations/status.upb.h"
#include "udpa/annotations/versioning.upb.h"
#include "validate/validate.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout *const envoy_config_cluster_v3_OutlierDetection_submsgs[2] = {
  &google_protobuf_Duration_msginit,
  &google_protobuf_UInt32Value_msginit,
};

static const upb_msglayout_field envoy_config_cluster_v3_OutlierDetection__fields[21] = {
  {1, UPB_SIZE(4, 8), 1, 1, 11, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(8, 16), 2, 0, 11, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(12, 24), 3, 0, 11, _UPB_MODE_SCALAR},
  {4, UPB_SIZE(16, 32), 4, 1, 11, _UPB_MODE_SCALAR},
  {5, UPB_SIZE(20, 40), 5, 1, 11, _UPB_MODE_SCALAR},
  {6, UPB_SIZE(24, 48), 6, 1, 11, _UPB_MODE_SCALAR},
  {7, UPB_SIZE(28, 56), 7, 1, 11, _UPB_MODE_SCALAR},
  {8, UPB_SIZE(32, 64), 8, 1, 11, _UPB_MODE_SCALAR},
  {9, UPB_SIZE(36, 72), 9, 1, 11, _UPB_MODE_SCALAR},
  {10, UPB_SIZE(40, 80), 10, 1, 11, _UPB_MODE_SCALAR},
  {11, UPB_SIZE(44, 88), 11, 1, 11, _UPB_MODE_SCALAR},
  {12, UPB_SIZE(3, 3), 0, 0, 8, _UPB_MODE_SCALAR},
  {13, UPB_SIZE(48, 96), 12, 1, 11, _UPB_MODE_SCALAR},
  {14, UPB_SIZE(52, 104), 13, 1, 11, _UPB_MODE_SCALAR},
  {15, UPB_SIZE(56, 112), 14, 1, 11, _UPB_MODE_SCALAR},
  {16, UPB_SIZE(60, 120), 15, 1, 11, _UPB_MODE_SCALAR},
  {17, UPB_SIZE(64, 128), 16, 1, 11, _UPB_MODE_SCALAR},
  {18, UPB_SIZE(68, 136), 17, 1, 11, _UPB_MODE_SCALAR},
  {19, UPB_SIZE(72, 144), 18, 1, 11, _UPB_MODE_SCALAR},
  {20, UPB_SIZE(76, 152), 19, 1, 11, _UPB_MODE_SCALAR},
  {21, UPB_SIZE(80, 160), 20, 0, 11, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_config_cluster_v3_OutlierDetection_msginit = {
  &envoy_config_cluster_v3_OutlierDetection_submsgs[0],
  &envoy_config_cluster_v3_OutlierDetection__fields[0],
  UPB_SIZE(88, 168), 21, false, 21, 255,
};

#include "upb/port_undef.inc"

