// Copyright 2021 The gRPC Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
#ifndef GRPC_CORE_LIB_IOMGR_EVENT_ENGINE_POLLSET_H
#define GRPC_CORE_LIB_IOMGR_EVENT_ENGINE_POLLSET_H

#include <grpc/support/port_platform.h>

#ifdef GRPC_USE_EVENT_ENGINE

/// Signals pollset_work that some work has been done.
void grpc_pollset_ee_broadcast_event();

#endif
#endif  // GRPC_CORE_LIB_IOMGR_EVENT_ENGINE_POLLSET_H
